import _extends from "@babel/runtime/helpers/esm/extends";
import { Children, cloneElement, createRef, forwardRef, useCallback, useEffect, useMemo, useReducer, useRef, useState } from 'react';
import PropTypes from 'prop-types';
import { getTarget } from '@s-ui/js/lib/react';
import { inputSizes as SELECT_SIZES } from '@s-ui/react-atom-input';
import useMergeRefs from '@s-ui/react-hooks/lib/useMergeRefs';
import { moleculeDropdownListSizes as SIZES } from '@s-ui/react-molecule-dropdown-list';
import MoleculeSelectMultipleSelection from './components/MultipleSelection.js';
import MoleculeSelectSingleSelection from './components/SingleSelection.js';
import { DropdownContext, ENABLED_KEYS, getClassName, getOptionData, SELECT_STATES, SELECTION_KEYS } from './config.js';
import { jsx as _jsx } from "react/jsx-runtime";
var useFunctionalRef = function useFunctionalRef() {
  return useReducer(function (_s, node) {
    return node;
  }, null);
};
var MoleculeSelect = /*#__PURE__*/forwardRef(function (props, forwardedRef) {
  var onBlur = props.onBlur,
    isOpen = props.isOpen,
    onToggle = props.onToggle,
    children = props.children,
    errorState = props.errorState,
    state = props.state,
    disabled = props.disabled,
    keysSelection = props.keysSelection,
    multiselection = props.multiselection,
    _props$hasSearch = props.hasSearch,
    hasSearch = _props$hasSearch === void 0 ? false : _props$hasSearch,
    searchIcon = props.searchIcon,
    searchPlaceholder = props.searchPlaceholder,
    noResults = props.noResults,
    refMoleculeSelectFromProps = props.refMoleculeSelect,
    ariaLabel = props['aria-label'],
    onSearch = props.onSearch;
  var refOptions = useRef({});
  var refMoleculeSelect = useRef(refMoleculeSelectFromProps);
  var refsMoleculeSelectOptions = useRef([]);
  var ref = useMergeRefs(forwardedRef, refMoleculeSelect);
  var _useFunctionalRef = useFunctionalRef(),
    inputSearch = _useFunctionalRef[0],
    setInputRef = _useFunctionalRef[1];
  var _useState = useState(isOpen || false),
    isOpenState = _useState[0],
    setIsOpenState = _useState[1];
  useEffect(function () {
    return setIsOpenState(isOpen);
  }, [isOpen, setIsOpenState]);
  Object.assign(refOptions.current, getOptionData(children));
  var _useState2 = useState(false),
    focus = _useState2[0],
    setFocus = _useState2[1];
  var extendedChildren = useMemo(function () {
    return Children.toArray(children).filter(Boolean).map(function (child, index) {
      refsMoleculeSelectOptions.current[index] = /*#__PURE__*/createRef();
      return /*#__PURE__*/cloneElement(child, {
        ref: refsMoleculeSelectOptions.current[index],
        selectKey: keysSelection
      });
    });
  }, [children, keysSelection]);
  var numOptions = Children.toArray(extendedChildren).length;
  var className = getClassName({
    state: state,
    errorState: errorState,
    focus: focus,
    disabled: disabled
  });
  var handleToggle = useCallback(function (ev, _temp) {
    var _ref = _temp === void 0 ? {
        isOutsideEvent: false
      } : _temp,
      isOpen = _ref.isOpen,
      isOutsideEvent = _ref.isOutsideEvent;
    setIsOpenState(isOpen !== undefined ? isOpen : !isOpenState);
    typeof onToggle === 'function' && onToggle(ev, {
      isOpen: isOpen
    });
    if (!isOutsideEvent) {
      ev.preventDefault();
      ev.stopPropagation();
    }
  }, [isOpenState, onToggle]);
  var closeList = useCallback(function (ev, _ref2) {
    var _ref2$isOutsideEvent = _ref2.isOutsideEvent,
      isOutsideEvent = _ref2$isOutsideEvent === void 0 ? false : _ref2$isOutsideEvent;
    handleToggle(ev, {
      isOpen: false,
      isOutsideEvent: isOutsideEvent
    });
  }, [handleToggle]);
  var handleOutsideClick = useCallback(function (ev) {
    if (disabled) return;
    if (refMoleculeSelect.current && !refMoleculeSelect.current.contains(ev.target)) {
      // outside click
      closeList(ev, {
        isOutsideEvent: true
      });
      setFocus(false);
    }
  }, [closeList, disabled]);
  var focusFirstOption = useCallback(function (ev) {
    var options = refsMoleculeSelectOptions.current.map(function (option) {
      return getTarget(option.current);
    });
    options[0] && options[0].focus();
    ev.preventDefault();
    ev.stopPropagation();
  }, [refsMoleculeSelectOptions]);
  var focusSearchInput = useCallback(function (ev) {
    var isEnabledKey = ENABLED_KEYS.includes(ev == null ? void 0 : ev.key);
    isEnabledKey ? focusFirstOption(ev) : inputSearch == null ? void 0 : inputSearch.focus();
    ev == null ? void 0 : ev.preventDefault();
    ev == null ? void 0 : ev.stopPropagation();
  }, [inputSearch, focusFirstOption]);
  var isFirstOptionFocused = useCallback(function () {
    var options = refsMoleculeSelectOptions.current.map(function (option) {
      return getTarget(option.current);
    });
    return document.activeElement === options[0];
  }, [refsMoleculeSelectOptions]);
  useEffect(function () {
    Object.assign(refOptions.current, getOptionData(children));
    document.addEventListener('touchend', handleOutsideClick);
    document.addEventListener('mousedown', handleOutsideClick);
    return function () {
      document.removeEventListener('touchend', handleOutsideClick);
      document.removeEventListener('mousedown', handleOutsideClick);
    };
  }, [children, handleOutsideClick]);
  useEffect(function () {
    isOpenState && setTimeout(function () {
      return focusSearchInput();
    });
  }, [isOpenState, focusSearchInput]);
  var handleKeyDown = function handleKeyDown(ev) {
    ev.persist();
    var isEnabledKey = ENABLED_KEYS.includes(ev.key);
    if (!isOpenState && isEnabledKey) {
      handleToggle(ev, {
        isOpen: !isOpenState
      });
      setTimeout(function () {
        return focusFirstOption(ev);
      });
    } else if (ev.key === 'Escape') {
      closeList(ev, {
        isOutsideEvent: true
      });
    } else {
      isOpenState && setTimeout(function () {
        return focusSearchInput(ev);
      });
      !hasSearch && setTimeout(function () {
        return focusFirstOption(ev);
      });
    }
  };
  var handleFocusOut = function handleFocusOut(event) {
    onBlur(event);
    setFocus(false);
  };
  var handleFocusIn = function handleFocusIn() {
    return !disabled && !hasSearch && setFocus(true);
  };
  var handleClick = function handleClick(ev) {
    ev.persist();
    if (focus) {
      !hasSearch && setTimeout(function () {
        return focusFirstOption(ev);
      });
    }
  };
  var Select = multiselection ? MoleculeSelectMultipleSelection : MoleculeSelectSingleSelection;
  var context = {
    hasSearch: hasSearch,
    setInputRef: setInputRef,
    isOpen: isOpenState,
    focusFirstOption: focusFirstOption,
    inputSearch: inputSearch,
    onSearch: onSearch,
    isFirstOptionFocused: isFirstOptionFocused,
    searchPlaceholder: searchPlaceholder,
    searchIcon: searchIcon
  };
  return /*#__PURE__*/_jsx(DropdownContext.Provider, {
    value: context,
    children: /*#__PURE__*/_jsx("div", {
      ref: ref,
      tabIndex: "0",
      className: className,
      onKeyDown: handleKeyDown,
      onFocus: handleFocusIn,
      onBlur: handleFocusOut,
      onClick: handleClick,
      "aria-label": ariaLabel,
      children: /*#__PURE__*/_jsx(Select, _extends({
        refMoleculeSelect: refMoleculeSelect,
        optionsData: refOptions.current,
        isOpen: isOpenState
      }, props, {
        onToggle: handleToggle,
        children: numOptions > 0 ? extendedChildren : noResults
      }))
    })
  });
});
MoleculeSelect.defaultProps = {
  onBlur: function onBlur() {},
  disabled: false,
  keysSelection: SELECTION_KEYS,
  onChange: function onChange() {},
  readOnly: false,
  selectSize: SELECT_SIZES.MEDIUM
};
MoleculeSelect.displayName = 'MoleculeSelect';
export default MoleculeSelect;
export { SIZES as moleculeSelectDropdownListSizes };
export { SELECT_SIZES as moleculeSelectSizes };
export { SELECT_STATES as moleculeSelectStates };